/*LINTLIBRARY*/

/*  @(#)items.c 1.21 93/03/30
 *
 *  Procedures for manipulating the reve game panel items.
 *
 *  Copyright (C) 1990 - 1993 - Rich Burridge & Yves Gallot.
 *  All rights reserved.
 *
 *  Permission is granted to copy this source, for redistribution
 *  in source form only, provided the news headers in "substantially
 *  unaltered format" are retained, the introductory messages are not
 *  removed, and no monies are exchanged.
 *
 *  Permission is also granted to copy this source, without the
 *  news headers, for the purposes of making an executable copy by
 *  means of compilation, provided that such copy will not be used
 *  for the purposes of competition in any othello tournaments, without
 *  prior permission from the authors.
 *
 *  No responsibility is taken for any errors on inaccuracies inherent
 *  either to the comments or the code of this program, but if reported
 *  (see README file), then an attempt will be made to fix them.
 */

#include "reve.h"
#include "color.h"
#include "extern.h"


void
draw_button(wtype, item, image)      /* Draw a panel button. */
enum win_type wtype ;
enum panel_type item ;
enum image_type image ;
{
  char *label ;
  int h, w, x, y ;
 
  x = items[(int) item].x ;
  y = items[(int) item].y ;
  if (x == -1 && y == -1) return ;
  h = BHEIGHT ;
  w = BWIDTH ;
  if (iscolor)
    {
      color_area(wtype, x, y, w, h, C_ITEMS) ;
      draw_rect(wtype, x, y, x+w, y+h, RSRC, C_BLACK) ;
      if (image == BUT_NORMAL)
        {
          draw_line(wtype, x+1, y+1,   x+w-2, y+1,   RSRC, C_ITEMS_L) ;
          draw_line(wtype, x+1, y+2,   x+w-3, y+2,   RSRC, C_ITEMS_L) ;
          draw_line(wtype, x+1, y+3,   x+w-4, y+3,   RSRC, C_ITEMS_L) ;
          draw_line(wtype, x+1, y+4,   x+1,   y+h-2, RSRC, C_ITEMS_L) ;
          draw_line(wtype, x+2, y+4,   x+2,   y+h-3, RSRC, C_ITEMS_L) ;
          draw_line(wtype, x+3, y+4,   x+3,   y+h-4, RSRC, C_ITEMS_L) ;

          draw_line(wtype, x+1, y+h-1, x+w-1, y+h-1, RSRC, C_ITEMS_D) ;
          draw_line(wtype, x+2, y+h-2, x+w-1, y+h-2, RSRC, C_ITEMS_D) ;
          draw_line(wtype, x+3, y+h-3, x+w-1, y+h-3, RSRC, C_ITEMS_D) ;
          draw_line(wtype, x+w-1, y+1, x+w-1, y+h-4, RSRC, C_ITEMS_D) ;
          draw_line(wtype, x+w-2, y+2, x+w-2, y+h-4, RSRC, C_ITEMS_D) ;
          draw_line(wtype, x+w-3, y+3, x+w-3, y+h-4, RSRC, C_ITEMS_D) ;
        }
      else if (image == BUT_INVERT)
        {
          draw_rect(wtype, x+1,   y+1,   x+w-1,   y+h-1, RSRC, C_BLACK) ;
          draw_line(wtype, x+2,   y+2,   x+w-2,   y+2,   RSRC, C_ITEMS_L) ;
          draw_line(wtype, x+2,   y+2,   x+2,     y+h-3, RSRC, C_ITEMS_L) ;
          draw_line(wtype, x+2,   y+h-2, x+w-2,   y+h-2, RSRC, C_ITEMS_D) ;
          draw_line(wtype, x+w-2, y+2,   x+w-2,   y+h-4, RSRC, C_ITEMS_D) ;
        }
    } 
  else draw_stencil(wtype, x, y, BWIDTH, BHEIGHT, RSRC, C_WHITE,
                    BUT_STENCIL, image) ;
  label = items[(int) item].text ;
  x += ((BWIDTH - get_strwidth(NFONT, label)) / 2) ;
  y += (font_heights[(int) NFONT] +
       (BHEIGHT - font_heights[(int) NFONT]) / 2) ;
  draw_text(wtype, x, y, NFONT, C_TEXT, label) ;
}


void
draw_choice(wtype, item, color)
enum win_type wtype ;
enum panel_type item ;
int color ;
{
  char *str ;
  int height, i, maxw, tcol, val, width, x, xoff, y ;

  tcol = B_COLOR(C_TEXT) ;
  if (strlen(items[(int) item].label))
    draw_text(items[(int) item].wtype,
              items[(int) item].lx,
              items[(int) item].ly + font_heights[(int) BFONT],
              BFONT, tcol, items[(int) item].label) ;

  maxw = 0 ;
  for (i = 0; i < items[(int) item].nopts; i++)
    {
      str = items[(int) item].options[i] ;
      if ((val = get_strwidth(NFONT, str)) > maxw) maxw = val ;
    }

  maxw += CHOICEGAP ;
  x = items[(int) item].x ;
  y = items[(int) item].y ;
  height = font_heights[(int) NFONT] * 2 ;
  width = items[(int) item].nopts * maxw ;
  color_area(wtype, x, y, width, height, color) ;
  draw_rect(wtype, x, y, x + width, y + height, RSRC, C_BLACK) ;

  items[(int) item].width = width ;
  items[(int) item].height = height ;

  for (i = 1; i < items[(int) item].nopts; i++)
    draw_line(W_PROPS, x + (i * maxw), y,
                       x + (i * maxw), y + height, RSRC, C_BLACK) ;

  for (i = 0; i < items[(int) item].nopts; i++)
    {
      str = items[(int) item].options[i] ;
      val = get_strwidth(NFONT, str) ;
      xoff = (maxw - val) / 2 ;
      draw_text(items[(int) item].wtype,
                x + (i * maxw) + xoff,
                y + font_heights[(int) NFONT] +
                   (font_heights[(int) NFONT] / 2), NFONT, tcol, str) ;
    }

  val = items[(int) item].value ;
  if (val < 0 || val >= items[(int) item].nopts) return ;
  draw_rect(wtype,
            x + (val * maxw) + 1, y + 1,
            x + ((val + 1) * maxw) - 1, y + height - 1, RSRC, C_BLACK) ;
}


void
draw_clock(wtype, item, state)
enum win_type wtype ;
enum panel_type item ;
int state ;
{
  char *label ;
  int color, h, w, x, y ;

  if (state) color = B_COLOR(C_TEXT) ;
  else       color = W_COLOR(C_PANEL) ;
  if (strlen(items[(int) item].label))
    draw_text(items[(int) item].wtype,
              items[(int) item].lx,
              items[(int) item].ly + font_heights[(int) NFONT],
              NFONT, color, items[(int) item].label) ;

  x = items[(int) item].x ;
  y = items[(int) item].y - 10 ;
  w = CLWIDTH ;
  h = CLHEIGHT ;
  color_area(W_PANEL, x + 1, y + 1, w - 2, h - 2, W_COLOR(C_PANEL)) ;
  if (state) color = C_BLACK ;
  else       color = W_COLOR(C_PANEL) ;
  draw_rect(W_PANEL, x, y, x + w, y + h, RSRC, color) ;

  label = items[(int) item].text ;
  color = C_TEXT ;
  x += ((CLWIDTH - get_strwidth(NFONT, label)) / 2) ;
  y += (font_heights[(int) NFONT] +
       (CLHEIGHT - font_heights[(int) NFONT]) / 2) ;
  if (state) draw_text(wtype, x, y, NFONT, color, label) ;
}


void
draw_cycle(wtype, item, color, image)
enum win_type wtype ;
enum panel_type item ;
int color ;
enum image_type image ;
{
  int x, y ;

  x = items[(int) item].x + 40 ;
  y = items[(int) item].y ;
  if (iscolor)
    {
      draw_stencil(wtype, x, y, CWIDTH, CHEIGHT, RSRC, color,
                   CY_STENCIL, CY_STENCIL) ;
      draw_image(wtype, x, y, CWIDTH, CHEIGHT, image) ;
    }
  else draw_stencil(wtype, x, y, CWIDTH, CHEIGHT, RSRC, C_WHITE,
                    CY_STENCIL, image) ;
}


void
draw_cycle_item(wtype, item, color, image)
enum win_type wtype ;
enum panel_type item ;
int color ;
enum image_type image ;
{
  char val[4] ;
  int tcol ;

  tcol = B_COLOR(C_TEXT) ;
  if (strlen(items[(int) item].label))
    draw_text(items[(int) item].wtype,
              items[(int) item].lx,
              items[(int) item].ly + font_heights[(int) BFONT],
              BFONT, tcol, items[(int) item].label) ;

  if (item == MAX_DEPTH && !items[(int) item].value) STRCPY(val, "off") ;
  else SPRINTF(val, "%d", items[(int) item].value) ;
  set_cycle(wtype, item, val) ;
  draw_cycle(wtype, item, color, image) ;
}


void
draw_filename(str)      /* Output filename in text field. */
char *str ;
{
  int y ;

  color_area(W_PANEL, tx + 1, ty + 1, tw - 2, th - 2, W_COLOR(C_PANEL)) ;
  y = ty + (font_heights[(int) NFONT] +
           (th - font_heights[(int) NFONT]) / 2) - 5 ;
  draw_text(W_PANEL, tx + 5, y, NFONT, B_COLOR(C_TEXT), str) ;
  draw_line(W_PANEL, tx + 5 + get_strwidth(NFONT, str), y + 5,
                     tx + 5 + get_strwidth(NFONT, str),
                     y + 5 - font_heights[(int) NFONT], RSRC, C_BLACK) ;
}


void
draw_textfield()
{
  int x1, y1 ;

  lsval = (cury < (BBORDER + (1*(BHEIGHT+BGAP)))) ? 'l' : 's' ;
  x1 = BBORDER + BWIDTH + (BGAP / 2) ;
  y1 = BBORDER + BHEIGHT + (BGAP / 2) ;
  color_area(W_PANEL, x1, y1,
             4 * (BWIDTH + BGAP), BHEIGHT + BGAP, W_COLOR(C_PANEL)) ;
  draw_rect(W_PANEL, x1, y1, x1 + (4 * (BWIDTH + BGAP)) - 1,
                    y1 + BHEIGHT + BGAP - 1, RSRC, C_BLACK) ;
  draw_text(W_PANEL, BBORDER + BWIDTH + BGAP, BBORDER + BHEIGHT + BGAP +
            (font_heights[(int) NFONT] +
            (BHEIGHT - font_heights[(int) NFONT]) / 2) - 5,
            BFONT, B_COLOR(C_TEXT), "File:") ;
  tx = BBORDER + BWIDTH + BGAP + (BWIDTH / 2) + 10 ;
  ty = BBORDER + BHEIGHT + BGAP ;
  tw = (BWIDTH / 2) - 10 + (2 * (BWIDTH + BGAP)) ;
  th = BHEIGHT ;
  draw_rect(W_PANEL, tx, ty, tx + tw, ty + th, RSRC, C_BLACK) ;
  draw_filename(gamefile) ;

  tinput = 1 ;
  items[(int) CANCEL_BUT].x = BBORDER + (4*(BWIDTH+BGAP)) ;
  items[(int) CANCEL_BUT].y = BBORDER + (1*(BHEIGHT+BGAP)) ;
  draw_button(W_PANEL, CANCEL_BUT, BUT_NORMAL) ;
}


void
draw_toggle(wtype, item, color)
enum win_type wtype ;
enum panel_type item ;
int color ;
{
  enum image_type image ;
  int tcol, x, y ;

  tcol = B_COLOR(C_TEXT) ;
  if (strlen(items[(int) item].label))
    draw_text(items[(int) item].wtype,
              items[(int) item].lx,
              items[(int) item].ly + font_heights[(int) BFONT],
              BFONT, tcol, items[(int) item].label) ;

  if (items[(int) item].value) image = TOGGLE_ON ;
  else                         image = TOGGLE_OFF ;

  x = items[(int) item].x ;
  y = items[(int) item].y ;
  if (iscolor)
    {
      draw_stencil(wtype, x, y, TICKWIDTH, TICKHEIGHT, RSRC, color,
                   CY_STENCIL, CY_STENCIL) ;
      draw_image(wtype, x, y, TICKWIDTH, TICKHEIGHT, image) ;
    }
  else draw_stencil(wtype, x, y, TICKWIDTH, TICKHEIGHT, RSRC, C_WHITE,
                    CY_STENCIL, image) ;

  if (strlen(items[(int) item].text))
    draw_text(items[(int) item].wtype,
              items[(int) item].x + TICKWIDTH + 10,
              items[(int) item].y + font_heights[(int) NFONT],
              NFONT, tcol, items[(int) item].text) ;
}


void
get_filename()     /* Extract next character for load/save filename. */
{
  char c, dummy[MAXLINE] ;
  int finished, len, nochars ;

  finished = 0 ;
  c = cur_ch ;
  if (c == ESCAPE)
    {
      remove_textfield() ;
      return ;
    }
  len = strlen(gamefile) ;
       if (c == BACKSPACE || c == DEL) gamefile[len-1] = '\0' ;
  else if (c == RETURN || c == LINEFEED) finished = 1 ;
  else if (c >= 32) STRNCAT(gamefile, &c, 1) ;
  len = strlen(gamefile) ;
  nochars = (len <= 20) ? len : 20 ;
  STRNCPY(dummy, &gamefile[len - nochars], nochars) ;
  dummy[nochars] = '\0' ;
  draw_filename(dummy) ;
 
  if (finished)
    {
           if (lsval == 'l') load_game() ;
      else if (lsval == 's') save_game() ;
      remove_textfield() ;
    }
}


void
init_item(ptype, wtype, type, lx, ly, label, x, y, width, height, text,
          value, options, nopts, func)
enum panel_type ptype ;
enum win_type wtype ;
enum item_type type ;
int lx, ly, x, y, width, height, value, nopts ;
char *label, *text, **options ;
void (*func)() ;
{
  int n = (int) ptype ;

  items[n].wtype   = wtype ;      /* Which window the item goes in. */
  items[n].type    = type ;       /* Item type. */
  items[n].lx      = lx ;         /* X position of the label for this item. */
  items[n].ly      = ly ;         /* Y position of the label for this item. */
  STRCPY(items[n].label, label) ;   /* Label for this item (possibly NULL). */
  items[n].x       = x ;          /* X position of this panel item. */
  items[n].y       = y ;          /* Y position of this panel item. */
  items[n].width   = width ;      /* Width of this panel item. */
  items[n].height  = height ;     /* Height of this panel item. */
  STRCPY(items[n].text, text) ;   /* Text string associated with this item. */
  items[n].value   = value ;      /* Current value of item. */
  items[n].options = options ;    /* Pointer to list of choice strings. */
  items[n].nopts   = nopts ;      /* Number of options in list. */
  items[n].func    = func ;       /* Function to obey for this panel item. */
}


void
init_items()
{
  init_item(LOAD_BUT, W_PANEL, P_BUTTON, 0, 0, "",      /* Load button. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (0*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Load", 0, (char **) NULL, 0, draw_textfield) ;

  init_item(MOVES_BUT, W_PANEL, P_BUTTON, 0, 0, "",     /* Moves? button. */
            BBORDER + (1*(BWIDTH+BGAP)), BBORDER + (0*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Moves?", 0, (char **) NULL, 0, show_all_moves) ;

  init_item(NEW_GAME_BUT, W_PANEL, P_BUTTON, 0, 0, "",  /* New games button. */
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (0*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "New game", 0, (char **) NULL, 0, new_game) ;
 
  init_item(HELP_BUT, W_PANEL, P_BUTTON, 0, 0, "",      /* Help button. */
            BBORDER + (3*(BWIDTH+BGAP)), BBORDER + (0*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Help...", 0, (char **) NULL, 0, do_help) ;

  init_item(REDO_BUT, W_PANEL, P_BUTTON, 0, 0, "",      /* Redo button. */
            BBORDER + (4*(BWIDTH+BGAP)), BBORDER + (0*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Redo", 0, (char **) NULL, 0, redo) ;

  init_item(PROPS_BUT, W_PANEL, P_BUTTON, 0, 0, "",     /* Props button. */
            BBORDER + (5*(BWIDTH+BGAP)), BBORDER + (0*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Props...", 0, (char **) NULL, 0, do_props) ;

  init_item(SAVE_BUT, W_PANEL, P_BUTTON, 0, 0, "",      /* Save button. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (1*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Save", 0, (char **) NULL, 0, draw_textfield) ;

  init_item(SUGGEST_BUT, W_PANEL, P_BUTTON, 0, 0, "",   /* Suggest button. */
            BBORDER + (1*(BWIDTH+BGAP)), BBORDER + (1*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Suggest", 0, (char **) NULL, 0, suggest) ;

  init_item(PRINT_BUT, W_PANEL, P_BUTTON, 0, 0, "",     /* Print button. */
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (1*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Print", 0, (char **) NULL, 0, print_game) ;

  init_item(STOP_BUT, W_PANEL, P_BUTTON, 0, 0, "",      /* Stop button. */
            -1, -1, BWIDTH, BHEIGHT, "Stop", 0, (char **) NULL, 0, do_stop) ;

  init_item(UNDO_BUT, W_PANEL, P_BUTTON, 0, 0, "",      /* Undo button. */
            BBORDER + (4*(BWIDTH+BGAP)), BBORDER + (1*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Undo", 0, (char **) NULL, 0, undo) ;

  init_item(CANCEL_BUT, W_PANEL, P_BUTTON, 0, 0, "",    /* Cancel button. */
            -1, -1, BWIDTH, BHEIGHT, "Cancel", 0, (char **) NULL, 0,
            remove_textfield) ;

  init_item(QUIT_BUT, W_PANEL, P_BUTTON, 0, 0, "",      /* Quit button. */
            BBORDER + (5*(BWIDTH+BGAP)), BBORDER + (1*(BHEIGHT+BGAP)),
            BWIDTH, BHEIGHT, "Quit", 0, (char **) NULL, 0, destroy_reve) ;

  init_item(BLACK_PLAYS, W_PANEL, P_MESSAGE,  /* Black player message item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (2*(BHEIGHT+BGAP)),
            "Black:",
            BBORDER + (1*(BWIDTH+BGAP)), BBORDER + (2*(BHEIGHT+BGAP)),
            (3*BWIDTH) + (2*BGAP), BHEIGHT, "", 0,
            (char **) NULL, 0, NULL) ;

  init_item(WHITE_PLAYS, W_PANEL, P_MESSAGE,  /* White player message item. */
            BBORDER + (3*(BWIDTH+BGAP)), BBORDER + (2*(BHEIGHT+BGAP)),
            "White:",
            BBORDER + (4*(BWIDTH+BGAP)), BBORDER + (2*(BHEIGHT+BGAP)),
            (3*BWIDTH) + (2*BGAP), BHEIGHT, "", 1,
            (char **) NULL, 0, NULL) ;

  init_item(BLACK_CLOCK, W_PANEL, P_CLOCK,  /* Black player time left item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (3*(BHEIGHT+BGAP)),
            "Time left:",
            BBORDER + (1*(BWIDTH+BGAP)), BBORDER + (3*(BHEIGHT+BGAP)),
            0, 0, "", 1, (char **) NULL, 0, NULL) ;

  init_item(WHITE_CLOCK, W_PANEL, P_CLOCK,  /* White player time left item. */
            BBORDER + (3*(BWIDTH+BGAP)), BBORDER + (3*(BHEIGHT+BGAP)),
            "Time left:",
            BBORDER + (4*(BWIDTH+BGAP)), BBORDER + (3*(BHEIGHT+BGAP)),
            0, 0, "", 1, (char **) NULL, 0, NULL) ;

  init_item(PANEL_MES, W_PANEL, P_MESSAGE,          /* Panel message item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (4*(BHEIGHT+BGAP)), "",
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (4*(BHEIGHT+BGAP)),
            (6*BWIDTH) + (5*BGAP), BHEIGHT,
            "Use left mouse button to move", 0, (char **) NULL, 0, NULL) ;

  init_item(EVAL_MES, W_PANEL, P_MESSAGE,          /* Notes message item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (5*(BHEIGHT+BGAP)), "",
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (5*(BHEIGHT+BGAP)),
            (6*BWIDTH) + (5*BGAP), BHEIGHT, "", 0,
            (char **) NULL, 0, NULL) ;

  init_item(SCORE_MES, W_PANEL, P_MESSAGE,          /* Score message item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (6*(BHEIGHT+BGAP)), "",
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (6*(BHEIGHT+BGAP)),
            (3*BWIDTH) + (2*BGAP), BHEIGHT,
            "Black:  2, White:  2", 0, (char **) NULL, 0, NULL) ;

  init_item(TURN_MES, W_PANEL, P_MESSAGE,          /* Turn message item. */
            BBORDER + (3*(BWIDTH+BGAP)), BBORDER + (6*(BHEIGHT+BGAP)), "",
            BBORDER + (3*(BWIDTH+BGAP)), BBORDER + (6*(BHEIGHT+BGAP)),
            (3*BWIDTH) + (2*BGAP), BHEIGHT, "", 0,
            (char **) NULL, 0, NULL) ;

  init_item(COMP_CHOICE, W_PROPS, P_CHOICE,  /* Computer plays choice item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (0*(BHEIGHT+BGAP)),
            "Computer plays:",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (0*(BHEIGHT+BGAP)),
            0, 0, "", 0, (char **) comp_plays, 4, set_option) ;

  init_item(DIFF_CHOICE, W_PROPS, P_CHOICE,  /* Difficulty choice item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (1*(BHEIGHT+BGAP)),
            "Difficulty:",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (1*(BHEIGHT+BGAP)),
            0, 0, "", 0, (char **) diff_values, 9, set_option) ;

  init_item(MAX_DEPTH, W_PROPS, P_CYCLE,   /* Set search depth cycle item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (2*(BHEIGHT+BGAP)),
            "Set search depth:",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (2*(BHEIGHT+BGAP)),
            0, 0, "", 0, (char **) NULL, 0, set_option) ;

  init_item(OPT_ANIM, W_PROPS, P_TOGGLE,   /* Options (Animate Move) item. */
            BBORDER + (0*(BWIDTH+BGAP)), BBORDER + (3*(BHEIGHT+BGAP)),
            "Options:",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (3*(BHEIGHT+BGAP)),
            TICKWIDTH, TICKHEIGHT, "Animate Move", 0,
            (char **) NULL, 0, set_option) ;

  init_item(OPT_BEST, W_PROPS, P_TOGGLE,  /* Options (Show Current...) item. */
            0, 0, "",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (4*(BHEIGHT+BGAP)),
            TICKWIDTH, TICKHEIGHT, "Show Current Best Move", 0,
            (char **) NULL, 0, set_option) ;

  init_item(OPT_LAST, W_PROPS, P_TOGGLE,  /* Options (Show Last Move) item. */
            0, 0, "",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (5*(BHEIGHT+BGAP)),
            TICKWIDTH, TICKHEIGHT, "Show Last Move", 0,
            (char **) NULL, 0, set_option) ;

  init_item(OPT_EVAL, W_PROPS, P_TOGGLE,   /* Options (Show Eval...) item. */
            0, 0, "",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (6*(BHEIGHT+BGAP)),
            TICKWIDTH, TICKHEIGHT, "Show Evaluation Info.", 0,
            (char **) NULL, 0, set_option) ;

  init_item(OPT_NUM, W_PROPS, P_TOGGLE,   /* Options (Number Last...) item. */
            0, 0, "",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (7*(BHEIGHT+BGAP)),
            TICKWIDTH, TICKHEIGHT, "Number Last Move", 0,
            (char **) NULL, 0, set_option) ;

  init_item(OPT_FLIP, W_PROPS, P_TOGGLE,   /* Options (Don't Show...) item. */
            0, 0, "",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (8*(BHEIGHT+BGAP)),
            TICKWIDTH, TICKHEIGHT, "Don't Show Flip", 0,
            (char **) NULL, 0, set_option) ;

  init_item(OPT_MOVE, W_PROPS, P_TOGGLE,  /* Options (Invalid Move...) item. */
            0, 0, "",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (9*(BHEIGHT+BGAP)),
            TICKWIDTH, TICKHEIGHT, "Invalid Move Shows Legal Moves", 0,
            (char **) NULL, 0, set_option) ;

  init_item(OPT_CLK, W_PROPS, P_TOGGLE,  /* Options (Use clocks...) item. */
            0, 0, "",
            BBORDER + (2*(BWIDTH+BGAP)), BBORDER + (10*(BHEIGHT+BGAP)),
            TICKWIDTH, TICKHEIGHT, "Use Clocks to Time Moves", 0,
            (char **) NULL, 0, set_option) ;

  init_item(HELP_PAGE, W_HELP, P_CYCLE,     /* Help window page cycle. */
            BBORDER + (0*(BWIDTH+BGAP)), CGAP, "Page",
            BBORDER + (1*(BWIDTH+BGAP)), CGAP, 0, 0, "", 1,
            (char **) NULL, 0, display_help) ;
}


void
load_and_move()
{
  if (loadgame)
    {
      loadgame = 0 ;
      load_game() ;
    }

  if (first_move)
    {
      first_move = 0 ;
      next_player = BLACK ;
      opponent_move(next_player) ;
    }
}


void
make_message(wtype, item)         /* Display a message panel. */
enum win_type wtype ;
enum panel_type item ;
{
  char *message ;
  int bfh, h, ix, iy, lx, ly, nfh, w, y ;

  bfh     = font_heights[(int) BFONT] ;
  nfh     = font_heights[(int) NFONT] ;
  lx      = items[(int) item].lx ;
  ly      = items[(int) item].ly ;
  ix      = items[(int) item].x ;
  iy      = items[(int) item].y ;
  h       = items[(int) item].height ;
  w       = items[(int) item].width ;
  message = items[(int) item].text ;
  color_area(wtype, lx, ly, w, h, W_COLOR(C_ITEMS)) ;
  draw_rect(wtype, lx, ly, lx+w, ly+h, RSRC, C_BLACK) ;
  if (strlen(items[(int) item].label))
    {
      y = ly + bfh + ((BHEIGHT - bfh) / 2) ;
      draw_text(items[(int) item].wtype, lx+5, y,
                BFONT, B_COLOR(C_TEXT), items[(int) item].label) ;
    }

  y = iy + nfh + ((BHEIGHT - nfh) / 2) ;
  draw_text(wtype, ix+5 , y, NFONT, B_COLOR(C_TEXT), message) ;
}


void
message(mtype, str)
enum panel_type mtype ;
char *str ;
{
  if (str == NULL) return ;
  if (!started && mtype == PANEL_MES)
    FPRINTF(stderr, "%s: %s\n", progname, str) ;

  STRCPY(items[(int) mtype].text, str) ;
  make_message(W_PANEL, mtype) ;
}


void
paint_all()
{
  paint_board() ;
  paint_panel() ;
  if (help_showing)  set_frame(W_HELP,  help_showing) ;
  if (props_showing) set_frame(W_PROPS, props_showing) ;
  load_and_move() ;
}


void
paint_board()
{
  char s1[2], sa[2] ;
  int adjust, d, i, n, x, y ;

  color_area(W_BOARD, 0, 0, board_width, board_height, C_WHITE) ;
  if (iscolor)
    {
      color_area(W_BOARD, 0, 0, board_width, board_height, C_BORDER) ;
      color_area(W_BOARD, bborder, bborder, BOARD_SIZE * cell_width,
                 BOARD_SIZE * cell_height, C_SQUARE) ;
    }

  batch(IS_ON) ;
  for (n = 0; n <= BOARD_SIZE; n++)
    {
      color = B_COLOR(C_GRID) ;
      if (n == 0 || n == BOARD_SIZE) color = C_BLACK ;
      if (color == C_GRID) adjust = 1 ;
      else adjust = 0 ;
      draw_line(W_BOARD, (n * cell_width) + bborder,         /* Vertical. */
                bborder + adjust, (n * cell_width) + bborder,
                bborder + (BOARD_SIZE * cell_height) - adjust, RSRC, color) ;
      draw_line(W_BOARD, bborder + adjust,                  /* Horizontal. */
                bborder + (n * cell_height),
                bborder + (BOARD_SIZE * cell_width) - adjust,
                bborder + (n * cell_height), RSRC, color) ;
    }  
  batch(IS_OFF) ;
  batch(IS_ON) ;
  for (i = 0; i < BOARD_SIZE; i++)
    {
      color = (iscolor) ? C_WHITE : C_BLACK ;
      SPRINTF(s1, "%c", 'a' + i) ;
      draw_text(W_BOARD, bborder + (i * cell_width) + (cell_width / 2) - 2,
                bborder - 4, GFONT, color, s1) ;
      draw_text(W_BOARD, bborder + (i * cell_width) + (cell_width / 2) - 2,
                (2 * bborder) + (BOARD_SIZE * cell_height) - 2,
                GFONT, color, s1) ;
      SPRINTF(sa, "%c", '1' + i) ;
      draw_text(W_BOARD, bborder / 2 - 2,
                bborder + (i * cell_height) + (cell_height / 2) + 2,
                GFONT, color, sa) ;
      draw_text(W_BOARD, bborder + (BOARD_SIZE * cell_width) + (bborder/2) - 2,
                bborder + (i * cell_height) + (cell_height / 2) + 2,
                GFONT, color, sa) ;
    }
  batch(IS_OFF) ;
  batch(IS_ON) ;
  FOR_BOARD(i)
    if (board.square[i])
      {
        get_xy(i, &x, &y) ;
        draw_piece(board.square[i], x, y, RSRC) ;
      }
  if (DO_LAST)   show_last(last_move, IS_ON) ;
  if (DO_NUMBER) show_number(last_move, 60 - board.moves_left, IS_ON) ;

  if (show_moves || (invalid == TRUE && SHOW_LEGAL)) show_all(IS_ON) ;
  if (suggestion != -1) draw_symbol(S_SUGGEST, suggestion, IS_ON) ;
  batch(IS_OFF) ;
}


void
paint_help()
{
  if (hfp == NULL) return ;
  color_area(W_HELP, 0, 0,
             help_width, (2 * CGAP) + CHEIGHT, W_COLOR(C_PANEL)) ;
  draw_cycle_item(W_HELP, HELP_PAGE, C_ITEMS, CY_NORMAL) ;
  draw_line(W_HELP, 0,          (2 * CGAP) + CHEIGHT,
                    help_width, (2 * CGAP) + CHEIGHT, RSRC, C_BLACK) ;
  paint_help_text() ;
}


void
paint_help_text()
{
  char buf[MAXLINE] ;
  int i, y ;

  color_area(W_HELP, 0, (2 * CGAP) + CHEIGHT + 1, help_width,
             ((font_heights[(int) HFONT] + 1) * HELP_ROWS) + (2 * CGAP) - 1,
             W_COLOR(C_PANEL)) ;
  FSEEK(hfp, help_offsets[items[(int) HELP_PAGE].value], 0) ;
  for (i = 0; i < HELP_ROWS; i++)
    {
      if (fgets(buf, MAXLINE, hfp) == NULL) return ;
      y = (3 * CGAP) + CHEIGHT + ((i + 1) * (font_heights[(int) HFONT] + 1)) ;
      draw_text(W_HELP, CGAP, y, HFONT, B_COLOR(C_TEXT), buf) ;
    }
}


void
paint_panel()
{
  char mes[MAXLINE] ;

  if (iscolor)
    color_area(W_PANEL, 0, 0, PANEL_WIDTH, PANEL_HEIGHT, C_PANEL) ;

  draw_button(W_PANEL, LOAD_BUT,     BUT_NORMAL) ;
  draw_button(W_PANEL, MOVES_BUT,    BUT_NORMAL) ;
  draw_button(W_PANEL, NEW_GAME_BUT, BUT_NORMAL) ;
  draw_button(W_PANEL, HELP_BUT,     BUT_NORMAL) ;
  draw_button(W_PANEL, REDO_BUT,     BUT_NORMAL) ;
  draw_button(W_PANEL, PROPS_BUT,    BUT_NORMAL) ;

  draw_button(W_PANEL, SAVE_BUT,     BUT_NORMAL) ;
  draw_button(W_PANEL, SUGGEST_BUT,  BUT_NORMAL) ;
  draw_button(W_PANEL, PRINT_BUT,    BUT_NORMAL) ;
  draw_button(W_PANEL, STOP_BUT,     BUT_NORMAL) ;
  draw_button(W_PANEL, UNDO_BUT,     BUT_NORMAL) ;
  draw_button(W_PANEL, QUIT_BUT,     BUT_NORMAL) ;

  make_message(W_PANEL, BLACK_PLAYS) ;
  make_message(W_PANEL, WHITE_PLAYS) ;

  draw_clock(W_PANEL, BLACK_CLOCK, DO_CLOCK) ;
  draw_clock(W_PANEL, WHITE_CLOCK, DO_CLOCK) ;

  make_message(W_PANEL, PANEL_MES) ;
  make_message(W_PANEL, EVAL_MES) ;
  make_message(W_PANEL, SCORE_MES) ;
  make_message(W_PANEL, TURN_MES) ;

  if (tinput) draw_textfield() ;
  set_score() ;
  SPRINTF(mes, "%s to move.",
          (next_player == BLACK) ? bstone_name : wstone_name) ;
  message(TURN_MES, mes) ;
}


void
paint_prop_sheet()
{
  int color ;

  color = W_COLOR(C_PANEL) ;
  color_area(W_PROPS, 0, 0, PROPS_WIDTH, PROPS_HEIGHT, color) ;

  draw_choice(W_PROPS, COMP_CHOICE, color) ;
  draw_choice(W_PROPS, DIFF_CHOICE, color) ;

  draw_cycle_item(W_PROPS, MAX_DEPTH, C_ITEMS, CY_NORMAL) ;

  draw_toggle(W_PROPS, OPT_ANIM, color) ;
  draw_toggle(W_PROPS, OPT_BEST, color) ;
  draw_toggle(W_PROPS, OPT_LAST, color) ;
  draw_toggle(W_PROPS, OPT_EVAL, color) ;
  draw_toggle(W_PROPS, OPT_NUM,  color) ;
  draw_toggle(W_PROPS, OPT_FLIP, color) ;
  draw_toggle(W_PROPS, OPT_MOVE, color) ;
  draw_toggle(W_PROPS, OPT_CLK,  color) ;
}


void
remove_textfield()
{
  int x1, y1 ;

  x1 = BBORDER + BWIDTH + (BGAP / 2) ;
  y1 = BBORDER + BHEIGHT + (BGAP / 2) ;
  color_area(W_PANEL, x1, y1, 4 * (BWIDTH + BGAP), BHEIGHT + BGAP,
             W_COLOR(C_PANEL)) ;
  tinput = 0 ;
  items[(int) CANCEL_BUT].x = -1 ;
  items[(int) CANCEL_BUT].y = -1 ;

  draw_button(W_PANEL, SUGGEST_BUT, BUT_NORMAL) ;
  draw_button(W_PANEL, PRINT_BUT,   BUT_NORMAL) ;
  draw_button(W_PANEL, STOP_BUT,    BUT_NORMAL) ;
  draw_button(W_PANEL, UNDO_BUT,    BUT_NORMAL) ;
}


void
set_cycle(wtype, item, str)
enum win_type wtype ;
enum panel_type item ;
char *str ;
{
  color_area(wtype, items[(int) item].x, items[(int) item].y,
             items[(int) item].width, CHEIGHT, W_COLOR(C_PANEL)) ;
  draw_text(wtype, items[(int) item].x,
            items[(int) item].y + font_heights[(int) NFONT],
            NFONT, B_COLOR(C_TEXT), str) ;
  STRCPY(items[(int) item].text, str) ;
  items[(int) item].width = get_strwidth(NFONT, str) ;
}
